/*
  EditHighlight: Syntax highlighting for the PmWiki edit form
  Author (c) 2021 Petko Yotov https://www.pmwiki.org/support
  License: GNU GPLv2 or any more recent version released by the FSF
  Version: 20211123
*/

document.addEventListener('DOMContentLoaded', function(){
  var text = document.querySelector('#wikiedit textarea#text');
  if(!text) {return;}
  
  var lastTextContent = false;
  var GUIEditInterval = false;
  var resizeObserver;
  
  function updatePre() {
    if(! chk_hlt.checked) return;
    var tc = text.value;
    if(tc===lastTextContent) return;
    var t2 = tc+'\n';
    
    var obj = hljs.highlight(t2, {language: 'pmwiki'});
    htext.innerHTML = obj.value
    lastTextContent = tc;
  }
  function textScrolled() {
    if(! chk_hlt.checked) return;
    if(ignoreTextScrolled) return;

    if(ignorePreScrolled) clearTimeout(ignorePreScrolled-1);
    ignorePreScrolled = 1 + setTimeout(nullIPS, 100);
    htext.scrollTop = text.scrollTop;
    htext.scrollLeft = text.scrollLeft;
  }
  var ignoreTextScrolled = false, ignorePreScrolled = false;
  function preScrolled() { // browser's in-page search
    if(! chk_hlt.checked) return;
    if(ignorePreScrolled) return;
    if(ignoreTextScrolled) clearTimeout(ignoreTextScrolled-1);
    ignoreTextScrolled = 1 + setTimeout(nullITS, 100);
    text.scrollTop = htext.scrollTop;
    text.scrollLeft = htext.scrollLeft;
  }
  function nullITS(){ignoreTextScrolled = false;}
  function nullIPS(){ignorePreScrolled = false;}

  function resizePre() {
    if(! chk_hlt.checked) return;
    var rect = text.getBoundingClientRect();
    htext.style.width = rect.width + 'px';
    htext.style.height = rect.height + 'px';
    textScrolled();
  }

  function initPre() {
    text.insertAdjacentHTML('beforebegin', '<div id="hwrap"><pre id="htext" class="hljs language-pmwiki"></pre></div>');
    updatePre();
    resizePre();

    htext.addEventListener('scroll', preScrolled);
    text.addEventListener('scroll', textScrolled);
    text.addEventListener('input', updatePre);
    GUIEditInterval = setInterval(updatePre, 100); // for GUIEdit buttons

    resizeObserver = new ResizeObserver(resizePre)
    resizeObserver.observe(text);
  }

  function EnableHighlight(e) {
    chk_hlt_wrap.dataset.checked = chk_hlt.checked;
    if(chk_hlt.checked) {
      localStorage.setItem('EnableHighlight', 1);
      updatePre();
      resizePre();
    }
    else {
      lastTextContent = false;
      localStorage.removeItem('EnableHighlight');
    }
  }

  function initCheckbox(){
    if (typeof hljs == 'undefined') return;
    if(!hljs.getLanguage('pmwiki')) return setTimeout(initCheckbox, 100);

    var form = text.closest('form');
    form.insertAdjacentHTML('afterbegin', '<span id="chk_hlt_wrap">'
      +'<input type="checkbox" name="chk_hlt" id="chk_hlt"/><label for="chk_hlt"> Highlight</label></span>');

    initPre();
    var enabled = localStorage.getItem('EnableHighlight');
    if(enabled) {
      chk_hlt.checked = true;
      EnableHighlight();
    }
    chk_hlt.addEventListener('change', EnableHighlight);
  }

  initCheckbox();
});
